@@ -49,8 +49,6 @@ collecting template definitions.
 
 import argparse
 import bz2
-import codecs
-import fileinput
 import logging
 import os.path
 import re  # TODO use regex when it will be standard
@@ -59,12 +57,12 @@ from io import StringIO
 from multiprocessing import Queue, Process, cpu_count
 from timeit import default_timer
 
-from .extract import Extractor, ignoreTag
+from .extract import Extractor, ignoreTag, define_template, acceptedNamespaces
 
 # ===========================================================================
 
 # Program version
-version = '3.0'
+__version__ = '3.0.4'
 
 ##
 # Defined in <siteinfo>
@@ -113,7 +111,7 @@ modules = {
 # Output
 
 
-class NextFile(object):
+class NextFile():
 
     """
     Synchronous generation of next available file name.
@@ -137,14 +135,14 @@ class NextFile(object):
 
     def _dirname(self):
         char1 = self.dir_index % 26
-        char2 = self.dir_index / 26 % 26
+        char2 = int(self.dir_index / 26) % 26
         return os.path.join(self.path_name, '%c%c' % (ord('A') + char2, ord('A') + char1))
 
     def _filepath(self):
         return '%s/wiki_%02d' % (self._dirname(), self.file_index)
 
 
-class OutputSplitter(object):
+class OutputSplitter():
 
     """
     File-like object, that splits output to multiple files of a given max size.
@@ -198,12 +196,13 @@ def load_templates(file, output_file=None):
     global moduleNamespace, modulePrefix
     modulePrefix = moduleNamespace + ':'
     articles = 0
+    templates = 0
     page = []
     inText = False
     if output_file:
-        output = codecs.open(output_file, 'wb', 'utf-8')
+        output = open(output_file, 'w')
     for line in file:
-        line = line.decode('utf-8')
+        #line = line.decode('utf-8')
         if '<' not in line:  # faster than doing re.search()
             if inText:
                 page.append(line)
@@ -238,15 +237,16 @@ def load_templates(file, output_file=None):
             # FIXME: should reconstruct also moduleNamespace
             if title.startswith(templatePrefix):
                 define_template(title, page)
+                templates += 1
             # save templates and modules to file
             if output_file and (title.startswith(templatePrefix) or
                                 title.startswith(modulePrefix)):
                 output.write('<page>\n')
-                output.write('   <title>%s</title>\n' % title)
+                output.write('   <title>%s</title>\n' % title.encode('utf-8'))
                 output.write('   <ns>10</ns>\n')
                 output.write('   <text>')
                 for line in page:
-                    output.write(line)
+                    output.write(line.encode('utf-8'))
                 output.write('   </text>\n')
                 output.write('</page>\n')
             page = []
@@ -255,7 +255,22 @@ def load_templates(file, output_file=None):
                 logging.info("Preprocessed %d pages", articles)
     if output_file:
         output.close()
-        logging.info("Saved %d templates to '%s'", len(templates), output_file)
+        logging.info("Saved %d templates to '%s'", templates, output_file)
+    return templates
+
+
+def decode_open(filename, mode='rt', encoding='utf-8'):
+    """
+    Open a file, decode and decompress, depending on extension `gz`, or 'bz2`.
+    """
+    ext = os.path.splitext(filename)[1]
+    if ext == '.gz':
+        import gzip
+        return gzip.open(filename, mode, encoding=encoding)
+    elif ext == '.bz2':
+        return bz2.open(filename, mode=mode, encoding=encoding)
+    else:
+        return open(filename, mode, encoding=encoding)
 
 
 def process_dump(input_file, template_file, out_file, file_size, file_compress,
@@ -273,14 +288,11 @@ def process_dump(input_file, template_file, out_file, file_size, file_compress,
     global templateNamespace, templatePrefix
     global moduleNamespace, modulePrefix
 
-    if input_file == '-':
-        input = sys.stdin
-    else:
-        input = fileinput.FileInput(input_file, openhook=fileinput.hook_compressed)
+    input = decode_open(input_file)
 
     # collect siteinfo
     for line in input:
-        line = line.decode('utf-8')
+        line = line #.decode('utf-8')
         m = tagRE.search(line)
         if not m:
             continue
@@ -306,19 +318,19 @@ def process_dump(input_file, template_file, out_file, file_size, file_compress,
         template_load_start = default_timer()
         if template_file and os.path.exists(template_file):
             logging.info("Preprocessing '%s' to collect template definitions: this may take some time.", template_file)
-            file = fileinput.FileInput(template_file, openhook=fileinput.hook_compressed)
-            load_templates(file)
+            file = decode_open(template_file)
+            templates = load_templates(file)
             file.close()
         else:
             if input_file == '-':
                 # can't scan then reset stdin; must error w/ suggestion to specify template_file
                 raise ValueError("to use templates with stdin dump, must supply explicit template-file")
             logging.info("Preprocessing '%s' to collect template definitions: this may take some time.", input_file)
-            load_templates(input, template_file)
+            templates = load_templates(input, template_file)
             input.close()
-            input = fileinput.FileInput(input_file, openhook=fileinput.hook_compressed)
+            input = decode_open(input_file)
         template_load_elapsed = default_timer() - template_load_start
-        logging.info("Loaded %d templates in %.1fs", len(templates), template_load_elapsed)
+        logging.info("Loaded %d templates in %.1fs", templates, template_load_elapsed)
 
     if out_file == '-':
         output = sys.stdout
@@ -350,7 +362,7 @@ def process_dump(input_file, template_file, out_file, file_size, file_compress,
     # start worker processes
     logging.info("Using %d extract processes.", process_count)
     workers = []
-    for _ in xrange(max(1, process_count)):
+    for _ in range(max(1, process_count)):
         extractor = Process(target=extract_process,
                             args=(jobs_queue, output_queue))
         extractor.daemon = True  # only live while parent process lives
@@ -368,7 +380,7 @@ def process_dump(input_file, template_file, out_file, file_size, file_compress,
     inText = False
     redirect = False
     for line in input:
-        line = line.decode('utf-8')
+        #line = line.decode('utf-8')
         if '<' not in line:  # faster than doing re.search()
             if inText:
                 page.append(line)
@@ -400,8 +412,8 @@ def process_dump(input_file, template_file, out_file, file_size, file_compress,
             page.append(line)
         elif tag == '/page':
             colon = title.find(':')
-            if (colon < 0 or title[:colon] in acceptedNamespaces) and id != last_id and \
-                    not redirect and not title.startswith(templateNamespace):
+            if (colon < 0 or (title[:colon] in acceptedNamespaces) and id != last_id and
+                    not redirect and not title.startswith(templateNamespace)):
                 job = (id, title, page, ordinal)
                 jobs_queue.put(job)  # goes to any available extract_process
                 last_id = id
@@ -531,13 +543,13 @@ def main():
     groupS.add_argument("-a", "--article", action="store_true",
                         help="analyze a file containing a single article (debug option)")
     groupS.add_argument("-v", "--version", action="version",
-                        version='%(prog)s ' + version,
+                        version='%(prog)s ' + __version__,
                         help="print program version")
 
     args = parser.parse_args()
 
     Extractor.keepLinks = args.links
-    Extractor.toHTML = args.html
+    Extractor.HtmlFormatting = args.html
     if args.html:
         Extractor.keepLinks = True
 
@@ -581,7 +593,7 @@ def main():
                     load_templates(file)
 
         with open(input_file) as file:
-            page = file.read().decode('utf-8')
+            page = file.read()#.decode('utf-8')
             m = re.search(r'<id>(.*)</id>', page)
             id = m.group(1) if m else 0
             m = re.search(r'<title>(.*)</title>', page)
